@@ -27,18 +27,20 @@ edition_deprecate <- function(in_edition, what, instead = NULL) {
     return()
   }
 
-  warn(c(
-    paste0("`", what, "` was deprecated in ", edition_name(in_edition), "."),
+  cli::cli_warn(c(
+    "{.code {what}} was deprecated in {edition_name(in_edition)}.",
     i = instead
   ))
 }
 
 edition_require <- function(in_edition, what) {
-  if (edition_get() >= in_edition || isTRUE(getOption("testthat.edition_ignore"))) {
+  if (
+    edition_get() >= in_edition || isTRUE(getOption("testthat.edition_ignore"))
+  ) {
     return()
   }
 
-  stop(paste0("`", what, "` requires ", edition_name(in_edition), "."))
+  cli::cli_abort("{.code {what}} requires {edition_name(in_edition)}.")
 }
 
 edition_name <- function(x) {
@@ -60,54 +62,14 @@ edition_name <- function(x) {
 #' @export
 #' @param x Edition Should be a single integer.
 #' @param .env Environment that controls scope of changes. For expert use only.
-#' @keywords internal
 local_edition <- function(x, .env = parent.frame()) {
-  stopifnot(is_zap(x) || (is.numeric(x) && length(x) == 1))
-  old <- edition_set(x)
-  withr::defer(edition_set(old), envir = .env)
-}
-
-edition_set <- function(x) {
-  env_poke(testthat_env, "edition", x)
+  check_number_whole(x, min = 2, max = 3)
+  local_bindings(edition = x, .env = the, .frame = .env)
 }
 
 
 #' @export
 #' @rdname local_edition
 edition_get <- function() {
-  if (env_has(testthat_env, "edition")) {
-    env_get(testthat_env, "edition", default = 2L)
-  } else {
-    find_edition(".")
-  }
-}
-
-
-find_dep_version <- function(name, path, package = NULL) {
-  desc <- find_description(path, package)
-  if (is.null(desc)) {
-    return(NULL)
-  }
-
-  deps <- desc$get_deps()
-  i <- match(name, deps[["package"]])
-  if (is_na(i)) {
-    return(NULL)
-  }
-
-  dep <- deps[[i, "version"]]
-  dep <- strsplit(dep, " ")[[1]]
-  if (!is_character(dep, 2) && !is_string(dep[[1]], ">=")) {
-    return(NULL)
-  }
-
-  dep[[2]]
-}
-has_dep <- function(name, path, package = NULL) {
-  !is.null(find_dep_version(name, path, package = package))
-}
-
-use_rlang_1_0 <- function() {
-  rlang::is_true(peek_option("testthat:::rlang_dep")) &&
-    is_installed("rlang", version = "0.99.0.9001")
+  the$edition %||% find_edition(".")
 }
