@@ -36,25 +36,23 @@
 #'     ```
 #'     Config/testthat/load-all: list(export_all = FALSE, helpers = FALSE)
 #'     ```
-#' @param wrap DEPRECATED
-#' @keywords internal
 #' @return A list (invisibly) containing data about the test results.
 #' @inheritParams with_reporter
 #' @inheritParams source_file
 #' @export
-test_dir <- function(path,
-                     filter = NULL,
-                     reporter = NULL,
-                     env = NULL,
-                     ...,
-                     load_helpers = TRUE,
-                     stop_on_failure = TRUE,
-                     stop_on_warning = FALSE,
-                     wrap = lifecycle::deprecated(),
-                     package = NULL,
-                     load_package = c("none", "installed", "source")
-                     ) {
-
+test_dir <- function(
+  path,
+  filter = NULL,
+  reporter = NULL,
+  env = NULL,
+  ...,
+  load_helpers = TRUE,
+  stop_on_failure = TRUE,
+  stop_on_warning = FALSE,
+  package = NULL,
+  load_package = c("none", "installed", "source"),
+  shuffle = FALSE
+) {
   load_package <- arg_match(load_package)
 
   start_first <- find_test_start_first(path, load_package, package)
@@ -66,11 +64,7 @@ test_dir <- function(path,
     start_first = start_first
   )
   if (length(test_paths) == 0) {
-    abort("No test files found")
-  }
-
-  if (!is_missing(wrap)) {
-    lifecycle::deprecate_stop("3.0.0", "test_dir(wrap = )")
+    cli::cli_abort("No test files found.")
   }
 
   want_parallel <- find_parallel(path, load_package, package)
@@ -95,7 +89,8 @@ test_dir <- function(path,
     stop_on_failure = stop_on_failure,
     stop_on_warning = stop_on_warning,
     load_package = load_package,
-    parallel = parallel
+    parallel = parallel,
+    shuffle = shuffle
   )
 }
 
@@ -116,13 +111,16 @@ test_dir <- function(path,
 #' test_file(path)
 #' test_file(path, desc = "some tests have warnings")
 #' test_file(path, reporter = "minimal")
-test_file <- function(path,
-                      reporter = default_compact_reporter(),
-                      desc = NULL,
-                      package = NULL,
-                      ...) {
+test_file <- function(
+  path,
+  reporter = default_compact_reporter(),
+  desc = NULL,
+  package = NULL,
+  shuffle = FALSE,
+  ...
+) {
   if (!file.exists(path)) {
-    stop("`path` does not exist", call. = FALSE)
+    cli::cli_abort("{.arg path} does not exist.")
   }
 
   test_files(
@@ -131,27 +129,26 @@ test_file <- function(path,
     test_paths = basename(path),
     reporter = reporter,
     desc = desc,
+    shuffle = shuffle,
     ...
   )
 }
 
-test_files <- function(test_dir,
-                       test_package,
-                       test_paths,
-                       load_helpers = TRUE,
-                       reporter = default_reporter(),
-                       env = NULL,
-                       stop_on_failure = FALSE,
-                       stop_on_warning = FALSE,
-                       desc = NULL,
-                       wrap = TRUE,
-                       load_package = c("none", "installed", "source"),
-                       parallel = FALSE,
-                       error_call = caller_env()) {
-
-  if (!isTRUE(wrap)) {
-    lifecycle::deprecate_stop("3.0.0", "test_dir(wrap = )")
-  }
+test_files <- function(
+  test_dir,
+  test_package,
+  test_paths,
+  load_helpers = TRUE,
+  reporter = default_reporter(),
+  env = NULL,
+  stop_on_failure = FALSE,
+  stop_on_warning = FALSE,
+  desc = NULL,
+  load_package = c("none", "installed", "source"),
+  parallel = FALSE,
+  shuffle = FALSE,
+  error_call = caller_env()
+) {
 
   # Must keep these two blocks in sync
   if (parallel) {
@@ -164,7 +161,8 @@ test_files <- function(test_dir,
       env = env,
       stop_on_failure = stop_on_failure,
       stop_on_warning = stop_on_warning,
-      load_package = load_package
+      load_package = load_package,
+      shuffle = shuffle
     )
   } else {
     test_files_serial(
@@ -178,52 +176,67 @@ test_files <- function(test_dir,
       stop_on_warning = stop_on_warning,
       desc = desc,
       load_package = load_package,
+      shuffle = shuffle,
       error_call = error_call
     )
   }
-
 }
 
-test_files_serial <- function(test_dir,
-                       test_package,
-                       test_paths,
-                       load_helpers = TRUE,
-                       reporter = default_reporter(),
-                       env = NULL,
-                       stop_on_failure = FALSE,
-                       stop_on_warning = FALSE,
-                       desc = NULL,
-                       wrap = TRUE,
-                       load_package = c("none", "installed", "source"),
-                       error_call = caller_env()) {
+test_files_serial <- function(
+  test_dir,
+  test_package,
+  test_paths,
+  load_helpers = TRUE,
+  reporter = default_reporter(),
+  env = NULL,
+  stop_on_failure = FALSE,
+  stop_on_warning = FALSE,
+  desc = NULL,
+  load_package = c("none", "installed", "source"),
+  shuffle = FALSE,
+  error_call = caller_env()
+) {
+  # Because load_all() called by test_files_setup_env() will have already
+  # loaded them. We don't want to rely on testthat's loading since that
+  # only affects the test environment and we want to keep the helpers
+  # loaded in the user's session.
+  load_package <- arg_match(load_package)
+  if (load_package == "source") {
+    load_helpers <- FALSE
+  }
 
   env <- test_files_setup_env(test_package, test_dir, load_package, env)
   # record testing env for mocks
   local_testing_env(env)
 
   test_files_setup_state(test_dir, test_package, load_helpers, env)
-  reporters <- test_files_reporter(reporter)
+  reporters <- test_files_reporter(reporter, "serial", desc = desc)
 
-  with_reporter(reporters$multi,
+  with_reporter(
+    reporters$multi,
     lapply(
       test_paths,
       test_one_file,
       env = env,
       desc = desc,
+      shuffle = shuffle,
       error_call = error_call
     )
   )
 
-  test_files_check(reporters$list$get_results(),
+  test_files_check(
+    reporters$list$get_results(),
     stop_on_failure = stop_on_failure,
     stop_on_warning = stop_on_warning
   )
 }
 
-test_files_setup_env <- function(test_package,
-                                 test_dir,
-                                 load_package = c("none", "installed", "source"),
-                                 env = NULL) {
+test_files_setup_env <- function(
+  test_package,
+  test_dir,
+  load_package = c("none", "installed", "source"),
+  env = NULL
+) {
   library(testthat)
 
   load_package <- arg_match(load_package)
@@ -244,7 +257,7 @@ test_files_setup_env <- function(test_package,
 }
 
 find_load_all_args <- function(path) {
-  default <- list(export_all = TRUE, helpers = FALSE)
+  default <- list(export_all = TRUE, helpers = TRUE)
 
   desc <- find_description(path)
   if (is.null(desc)) {
@@ -258,7 +271,7 @@ find_load_all_args <- function(path) {
 
   args <- parse_expr(args)
   if (!is_call(args, "list")) {
-    abort("`Config/testthat/load-all` must be a list.", call = NULL)
+    cli::cli_abort("{.field Config/testthat/load-all} must be a list.")
   }
 
   args <- as.list(args[-1])
@@ -269,11 +282,11 @@ find_load_all_args <- function(path) {
 }
 
 test_files_setup_state <- function(
-    test_dir,
-    test_package,
-    load_helpers,
-    env,
-    frame = parent.frame()
+  test_dir,
+  test_package,
+  load_helpers,
+  env,
+  frame = parent.frame()
 ) {
   # Define testing environment
   local_test_directory(test_dir, test_package, .env = frame)
@@ -288,45 +301,82 @@ test_files_setup_state <- function(
     source_test_helpers(".", env)
   }
   source_test_setup(".", env)
-  withr::defer(source_test_teardown(".", env), frame)      # old school
+  withr::defer(source_test_teardown(".", env), frame) # old school
 }
 
-test_files_reporter <- function(reporter, .env = parent.frame()) {
+test_files_reporter <- function(
+  reporter,
+  mode = c("serial", "parallel"),
+  desc = NULL,
+  frame = caller_env()
+) {
+  mode <- arg_match(mode)
+
+  # User selected reporter
+  user <- find_reporter(reporter)
+
+  # Reporter that collect test results
   lister <- ListReporter$new()
-  reporters <- list(
-    find_reporter(reporter),
-    lister, # track data
-    local_snapshotter("_snaps", fail_on_new = FALSE, .env = .env)
+
+  # Snapshot reporter
+  if (mode == "parallel") {
+    snap_base <- MainprocessSnapshotReporter
+  } else {
+    snap_base <- SnapshotReporter
+  }
+  snap <- local_snapshotter(
+    snap_base,
+    fail_on_new = on_ci(),
+    desc = desc,
+    frame = frame
   )
+
+  reporters <- compact(list(user, lister, snap))
   list(
-    multi = MultiReporter$new(reporters = compact(reporters)),
+    multi = MultiReporter$new(reporters = reporters),
     list = lister
   )
 }
 
-test_files_check <- function(results, stop_on_failure = TRUE, stop_on_warning = FALSE) {
+test_files_check <- function(
+  results,
+  stop_on_failure = TRUE,
+  stop_on_warning = FALSE
+) {
   if (stop_on_failure && !all_passed(results)) {
-    stop("Test failures", call. = FALSE)
+    cli::cli_abort(
+      "Test failures.",
+      call = NULL,
+      trace = data.frame()
+    )
   }
   if (stop_on_warning && any_warnings(results)) {
-    stop("Tests generated warnings", call. = FALSE)
+    cli::cli_abort(
+      "Tests generated warnings.",
+      call = NULL,
+      trace = data.frame()
+    )
   }
 
   invisible(results)
 }
 
-test_one_file <- function(path,
-                          env = test_env(),
-                          desc = NULL,
-                          error_call = caller_env()) {
+test_one_file <- function(
+  path,
+  env = test_env(),
+  desc = NULL,
+  shuffle = FALSE,
+  error_call = caller_env()
+) {
   reporter <- get_reporter()
-  on.exit(teardown_run(), add = TRUE)
+  withr::defer(teardown_run())
 
   reporter$start_file(path)
   source_file(
     path,
     env = env(env),
     desc = desc,
+    shuffle = shuffle,
     error_call = error_call
   )
   reporter$end_context_if_started()
@@ -345,7 +395,10 @@ test_one_file <- function(path,
 #' @export
 teardown_env <- function() {
   if (is.null(the$teardown_env)) {
-    abort("`teardown_env()` has not been initialized", .internal = TRUE)
+    cli::cli_abort(
+      "{.fn teardown_env} has not been initialized.",
+      .internal = TRUE
+    )
   }
 
   the$teardown_env
@@ -371,7 +424,14 @@ local_teardown_env <- function(frame = parent.frame()) {
 #' @return A character vector of paths
 #' @keywords internal
 #' @export
-find_test_scripts <- function(path, filter = NULL, invert = FALSE, ..., full.names = TRUE, start_first = NULL) {
+find_test_scripts <- function(
+  path,
+  filter = NULL,
+  invert = FALSE,
+  ...,
+  full.names = TRUE,
+  start_first = NULL
+) {
   files <- dir(path, "^test.*\\.[rR]$", full.names = full.names)
   files <- filter_test_scripts(files, filter, invert, ...)
   order_test_scripts(files, start_first)
@@ -391,7 +451,9 @@ filter_test_scripts <- function(files, filter = NULL, invert = FALSE, ...) {
 
 find_test_start_first <- function(path, load_package, package) {
   # Make sure we get the local package package if not "installed"
-  if (load_package != "installed") package <- NULL
+  if (load_package != "installed") {
+    package <- NULL
+  }
   desc <- find_description(path, package)
   if (is.null(desc)) {
     return(NULL)
@@ -406,7 +468,9 @@ find_test_start_first <- function(path, load_package, package) {
 }
 
 order_test_scripts <- function(paths, start_first) {
-  if (is.null(start_first)) return(paths)
+  if (is.null(start_first)) {
+    return(paths)
+  }
   filemap <- data.frame(
     stringsAsFactors = FALSE,
     base = sub("\\.[rR]$", "", sub("^test[-_\\.]?", "", basename(paths))),
