@@ -34,40 +34,38 @@
 #' })
 #' }
 test_that <- function(desc, code) {
-  check_string(desc)
+  local_description_push(desc)
 
   code <- substitute(code)
-  if (edition_get() >= 3) {
-    if (!is_call(code, "{")) {
-      warn(
-        "The `code` argument to `test_that()` must be a braced expression to get accurate file-line information for failures.",
-        class = "testthat_braces_warning"
-      )
-    }
-  }
-
-  local_test_context()
-  test_code(
-    desc,
-    code,
-    env = parent.frame(),
-    default_reporter = local_interactive_reporter()
-  )
+  test_code(code, parent.frame())
 }
 
 # Access error fields with `[[` rather than `$` because the
 # `$.Throwable` from the rJava package throws with unknown fields
-test_code <- function(test, code, env, default_reporter, skip_on_empty = TRUE) {
+test_code <- function(code, env, reporter = NULL, skip_on_empty = TRUE) {
+  # Must initialise interactive reporter before local_test_context()
+  reporter <- get_reporter() %||% local_interactive_reporter()
+  local_test_context()
 
   frame <- caller_env()
-  reporter <- get_reporter() %||% default_reporter
 
+  test <- test_description()
   if (!is.null(test)) {
     reporter$start_test(context = reporter$.context, test = test)
-    on.exit(reporter$end_test(context = reporter$.context, test = test))
+    withr::defer(reporter$end_test(context = reporter$.context, test = test))
   }
 
+  if (the$top_level_test) {
+    # Not strictly necessary but nice to reset the count
+    the$test_expectations <- 0
+    the$top_level_test <- FALSE
+    withr::defer(the$top_level_test <- TRUE)
+  }
+  # Used to skip if the test _and_ its subtests have no expectations
+  starting_expectations <- the$test_expectations
+
   ok <- TRUE
+  snapshot_skipped <- FALSE
 
   # @param debug_end How many frames should be skipped to find the
   #   last relevant frame call. Only useful for the DebugReporter.
@@ -88,60 +86,32 @@ test_code <- function(test, code, env, default_reporter, skip_on_empty = TRUE) {
     reporter$add_result(context = reporter$.context, test = test, result = e)
   }
 
-  # Any error will be assigned to this variable first
-  # In case of stack overflow, no further processing (not even a call to
-  # signalCondition() ) might be possible
-  test_error <- NULL
-
   expressions_opt <- getOption("expressions")
   expressions_opt_new <- min(expressions_opt + 500L, 500000L)
 
-  # If no handlers are called we skip: BDD (`describe()`) tests are often
-  # nested and the top level might not contain any expectations, so we need
-  # some way to disable
-  handled <- !skip_on_empty
-
   handle_error <- function(e) {
-    handled <<- TRUE
-    # First thing: Collect test error
-    test_error <<- e
+    the$test_expectations <- the$test_expectations + 1L
 
     # Increase option(expressions) to handle errors here if possible, even in
-    # case of a stack overflow.  This is important for the DebugReporter.
-    # Call options() manually, avoid withr overhead.
-    options(expressions = expressions_opt_new)
-    on.exit(options(expressions = expressions_opt), add = TRUE)
+    # case of a stack overflow. This is important for the DebugReporter.
+    local_options(expressions = expressions_opt_new)
 
     # Add structured backtrace to the expectation
     if (can_entrace(e)) {
       e <- cnd_entrace(e)
     }
 
-    test_error <<- e
-
-    # Error will be handled by handle_fatal() if this fails; need to do it here
-    # to be able to debug with the DebugReporter
     register_expectation(e, 2)
-
-    e[["handled"]] <- TRUE
-    test_error <<- e
+    invokeRestart("end_test")
   }
   handle_fatal <- function(e) {
-    handled <<- TRUE
-    # Error caught in handle_error() has precedence
-    if (!is.null(test_error)) {
-      e <- test_error
-      if (isTRUE(e[["handled"]])) {
-        return()
-      }
-    }
-
+    the$test_expectations <- the$test_expectations + 1L
     register_expectation(e, 0)
   }
   handle_expectation <- function(e) {
-    handled <<- TRUE
-    register_expectation(e, 6)
-    invokeRestart("continue_test")
+    the$test_expectations <- the$test_expectations + 1L
+    register_expectation(e, 7)
+    invokeRestart("muffle_expectation")
   }
   handle_warning <- function(e) {
     # When options(warn) < 0, warnings are expected to be ignored.
@@ -160,47 +130,66 @@ test_code <- function(test, code, env, default_reporter, skip_on_empty = TRUE) {
     }
 
     register_expectation(e, 5)
-
-    maybe_restart("muffleWarning")
+    tryInvokeRestart("muffleWarning")
   }
   handle_message <- function(e) {
     if (edition_get() < 3) {
-     maybe_restart("muffleMessage")
+      tryInvokeRestart("muffleMessage")
     }
   }
   handle_skip <- function(e) {
-    handled <<- TRUE
+    the$test_expectations <- the$test_expectations + 1L
 
     debug_end <- if (inherits(e, "skip_empty")) -1 else 2
     register_expectation(e, debug_end)
-    signalCondition(e)
+    invokeRestart("end_test")
+  }
+  handle_interrupt <- function(e) {
+    if (!is.null(test)) {
+      cat("\n")
+      cli::cli_inform(c("!" = "Interrupting test: {test}"))
+    }
   }
 
   test_env <- new.env(parent = env)
   old <- options(rlang_trace_top_env = test_env)[[1]]
-  on.exit(options(rlang_trace_top_env = old), add = TRUE)
+  withr::defer(options(rlang_trace_top_env = old))
 
   withr::local_options(testthat_topenv = test_env)
 
   before <- inspect_state()
-  tryCatch(
-    withCallingHandlers(
-      {
-        eval(code, test_env)
-        if (!handled && !is.null(test)) {
-          skip_empty()
-        }
-      },
-      expectation = handle_expectation,
-      skip =        handle_skip,
-      warning =     handle_warning,
-      message =     handle_message,
-      error =       handle_error
+  withRestarts(
+    tryCatch(
+      withCallingHandlers(
+        {
+          eval(code, test_env)
+          new_expectations <- the$test_expectations > starting_expectations
+          if (snapshot_skipped) {
+            skip("On CRAN")
+          } else if (!new_expectations && skip_on_empty) {
+            skip_empty()
+          }
+        },
+        expectation = handle_expectation,
+        packageNotFoundError = function(e) {
+          if (on_cran()) {
+            skip(paste0("{", e$package, "} is not installed."))
+          }
+        },
+        snapshot_on_cran = function(cnd) {
+          snapshot_skipped <<- TRUE
+          invokeRestart("muffle_cran_snapshot")
+        },
+        skip = handle_skip,
+        warning = handle_warning,
+        message = handle_message,
+        error = handle_error,
+        interrupt = handle_interrupt
+      ),
+      # some errors may need handling here, e.g., stack overflow
+      error = handle_fatal
     ),
-    # some errors may need handling here, e.g., stack overflow
-    error = handle_fatal,
-    # skip silently terminate code
-    skip  = function(e) {}
+    end_test = function() {}
   )
   after <- inspect_state()
 
@@ -213,3 +202,30 @@ test_code <- function(test, code, env, default_reporter, skip_on_empty = TRUE) {
 
   invisible(ok)
 }
+
+
+# Maintain a stack of descriptions
+local_description_push <- function(description, frame = caller_env()) {
+  check_string(description, call = frame)
+  local_description_set(c(the$description, description), frame = frame)
+}
+local_description_set <- function(
+  description = character(),
+  frame = caller_env()
+) {
+  check_character(description, call = frame)
+
+  old <- the$description
+  the$description <- description
+  withr::defer(the$description <- old, frame)
+
+  invisible(old)
+}
+
+test_description <- function(desc = the$description) {
+  if (length(desc) == 0) {
+    NULL
+  } else {
+    paste(desc, collapse = " / ")
+  }
+}
