@@ -1,6 +1,5 @@
-
 test_that("teardown error", {
-  skip("teardown errors are ignored")
+  skip_on_covr()
   withr::local_envvar(TESTTHAT_PARALLEL = "TRUE")
   err <- tryCatch(
     capture.output(suppressMessages(testthat::test_local(
@@ -9,6 +8,14 @@ test_that("teardown error", {
     ))),
     error = function(e) e
   )
-  expect_s3_class(err, "testthat_process_error")
-  expect_match(err$message, "Error in teardown", fixed = TRUE)
+  expect_s3_class(err$parent, "callr_error")
+  expect_match(
+    err$message,
+    "At least one parallel worker failed to run teardown"
+  )
+  expect_match(
+    err$parent$parent$parent$message,
+    "Error in teardown",
+    fixed = TRUE
+  )
 })
