@@ -30,12 +30,23 @@ test_that("empty lines are preserved", {
   expect_snapshot(f(), error = TRUE)
 })
 
+test_that("line-endings fixed before comparison", {
+  x <- "a\n\rb"
+  expect_snapshot(cat(x))
+})
+
 test_that("multiple outputs of same type are collapsed", {
   expect_snapshot({
     x <- 1
     y <- 1
-    {message("a"); message("b")}
-    {warning("a"); warning("b")}
+    {
+      message("a")
+      message("b")
+    }
+    {
+      warning("a")
+      warning("b")
+    }
   })
 })
 
@@ -50,12 +61,16 @@ test_that("can scrub output/messages/warnings/errors", {
   expect_snapshot(secret(), transform = redact, error = TRUE)
 
   # Or with an inline fun
-  expect_snapshot(print("secret"), transform = ~ gsub("secret", "****", .x))
+  expect_snapshot(print("secret"), transform = \(x) gsub("secret", "****", x))
 })
 
 test_that("always checks error status", {
   expect_error(expect_snapshot(stop("!"), error = FALSE))
-  expect_failure(expect_snapshot(print("!"), error = TRUE))
+  expect_snapshot_failure(expect_snapshot(print("!"), error = TRUE))
+})
+
+test_that("snapshots of failures fail", {
+  expect_snapshot_failure(expect_snapshot(fail()))
 })
 
 test_that("can capture error/warning messages", {
@@ -142,11 +157,86 @@ test_that("errors and warnings are folded", {
   expect_snapshot(error = TRUE, f())
 })
 
-test_that("hint is informative", {
-  local_reproducible_output(crayon = TRUE, hyperlinks = TRUE, rstudio = TRUE)
+# I don't know how to test this automatically; wrapping it in another
+# snapshot doesn't capture the behaviour I expected, presumably due to the
+# way that errors bubble up
+# test_that("errors in snapshots behave like regular errors", {
+#   f <- function() g()
+#   g <- function() h()
+#   h <- function() abort("!")
+#
+#   expect_snapshot(f())
+#   expect_snapshot(1 + 1)
+# })
 
-  expect_snapshot({
-    cat(snapshot_accept_hint("_default", "bar.R", reset_output = FALSE))
-    cat(snapshot_accept_hint("foo", "bar.R", reset_output = FALSE))
+test_that("hint is informative", {
+  local_mocked_bindings(in_check_reporter = function() FALSE)
+  withr::local_envvar(GITHUB_ACTIONS = "false", TESTTHAT_WD = NA)
+
+  expect_snapshot(snapshot_hint("bar.R", reset_output = FALSE))
+})
+
+test_that("hint includes path when WD is different", {
+  local_mocked_bindings(in_check_reporter = function() FALSE)
+  withr::local_envvar(TESTTHAT_WD = "..")
+
+  hint <- snapshot_hint("bar.R", reset_output = FALSE)
+  # Can't use snapshot here because its hint will get the wrong path
+  expect_match(
+    hint,
+    'snapshot_accept("bar.R", "testthat")',
+    fixed = TRUE,
+    all = FALSE
+  )
+})
+
+test_that("expect_snapshot requires a non-empty test label", {
+  local_description_set()
+  local_on_cran(FALSE)
+
+  test_that("", {
+    expect_error(expect_snapshot(1 + 1))
+  })
+
+  pass() # quiet message about this test being empty
+})
+
+test_that("expect_snapshot validates its inputs", {
+  expect_snapshot(error = TRUE, {
+    expect_snapshot(1 + 1, cran = "yes")
+    expect_snapshot(1 + 1, error = "yes")
+    expect_snapshot(1 + 1, cnd_class = "yes")
   })
 })
+
+test_that("expect_snapshot_output validates its inputs", {
+  expect_snapshot(error = TRUE, {
+    expect_snapshot_output(cat("test"), cran = "yes")
+  })
+})
+
+test_that("expect_snapshot_error validates its inputs", {
+  expect_snapshot(error = TRUE, {
+    expect_snapshot_error(stop("!"), class = 123)
+    expect_snapshot_error(stop("!"), cran = "yes")
+  })
+})
+
+test_that("expect_snapshot_warning validates its inputs", {
+  expect_snapshot(error = TRUE, {
+    expect_snapshot_warning(warning("!"), class = 123)
+    expect_snapshot_warning(warning("!"), cran = "yes")
+  })
+})
+
+test_that("on CRAN, snapshots generate skip at end of test", {
+  local_on_cran(TRUE)
+
+  expectations <- capture_expectations(test_that("", {
+    expect_snapshot(1 + 1)
+    expect_true(TRUE)
+  }))
+  expect_length(expectations, 2)
+  expect_s3_class(expectations[[1]], "expectation_success")
+  expect_s3_class(expectations[[2]], "expectation_skip")
+})
