@@ -7,5 +7,8 @@ test_that("can detect state changes", {
   set_state_inspector(function() list(x = getOption("x")))
   withr::defer(set_state_inspector(NULL))
 
-  expect_snapshot_reporter(CheckReporter$new(), test_path("reporters/state-change.R"))
+  expect_snapshot_reporter(
+    CheckReporter$new(),
+    test_path("reporters/state-change.R")
+  )
 })
